local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikItemLevel = BaikCore.BaikItemLevel
local ItemUpgradeInfo = LibStub("LibItemUpgradeInfo-1.0")
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local Log = BaikCore.Log
local Table = BaikCore.Table
local Ui = BaikCore.Ui

local Color = BaikItemLevel.Color
local OrderNumber = BaikItemLevel.OrderNumber

-- Add Module
local Badge = BaikItemLevel:NewModule("Badge", "AceTimer-3.0", "AceEvent-3.0")
BaikItemLevel.Badge = Badge

-- Constants
local INVALID = -1
local NO_EQUIPMENT = -2
local INVALID_COLOR = Ui:CreateColor(0.85, 0.209, 0.106, 1)

local ITEM_FILTER_DB = "ilvl_filter"
local NO_FILTER = 0

local NUMBER_PATTERN = "(-?%d+)"

-- Private Methods
local linkFunc = {}
local badges = {}
local missing = {}
local timer = nil

-- Private Methods
local function CreateText(slot)
    local badge = badges[slot]
    -- Badge alread exists
    if not badge then
        badge = slot:CreateFontString(string.format("%s%s", "Baik", slot:GetName()),
                                      "OVERLAY", "NumberFontNormal")
        badge:SetPoint("BOTTOM", 0, 3)
        badges[slot] = badge
    end

    return badge
end

local function GetItemLink(slot)
    local func = linkFunc[slot]

    if not func then
        return nil
    end

    return func(slot)
end

local function IsEquipment(link)
    if not link then
        return false
    end

    local itemType = select(6, GetItemInfo(link))

    return itemType == "Armor" or itemType == "Weapon"
end

local function GetItemLevel(link)
    if not IsEquipment(link) then
        return NO_EQUIPMENT
    end

    local filterLevel = Database:Get(ITEM_FILTER_DB)
    local itemLevel = ItemUpgradeInfo:GetUpgradedItemLevel(link)
    if filterLevel > NO_FILTER  and itemLevel < filterLevel then
        return NO_EQUIPMENT
    end

    return itemLevel and itemLevel or INVALID
end

local function InsertOpts(args)
    local filterHeader = {
        name = "iLvl Filter",
        type = "header",
        order = OrderNumber:GetNext(),
    }
    local filterNumber = {
        name = "Filter iLvl below",
        type = "input",
        pattern = NUMBER_PATTERN,
        usage = "Default (0 no filter)",
        order = OrderNumber:GetNext(),
        get = function()
            return tostring(Database:Get(ITEM_FILTER_DB))
        end,
        set = function(self, value, ...)
            local number = string.match(value, NUMBER_PATTERN)

            Database:Set(ITEM_FILTER_DB, tonumber(number))
            Badge:SendMessage(BaikItemLevel.MESSAGE)
        end
    }
    args["filter_header"] = filterHeader
    args["filter_number"] = filterNumber
end

-- Badge API
function Badge:Setup(slot, func)
    Assert:Table(slot)
    Assert:Function(func)

    if not badges[slot] then
        CreateText(slot)
    end
    linkFunc[slot] = func
end

function Badge:Update(slot)
    local badge = badges[slot]

    if not badge then
        return
    end

    local itemLink = GetItemLink(slot)
    local itemLevel = GetItemLevel(itemLink)

    if itemLevel == NO_EQUIPMENT then
        badge:Hide()
        return
    end

    badge:Show()

    if itemLevel == INVALID then
        badge:SetTextColor(INVALID_COLOR.r, INVALID_COLOR.g,
                           INVALID_COLOR.b, INVALID_COLOR.a)
        badge:SetText("load")
        table.insert(missing, slot)
        if not timer then
            timer = self:ScheduleTimer("UpdateMissing", 5)
        end
        return
    end

    badge:SetText(itemLevel)
    local color = Color:Get(itemLink, itemLevel)
    if not color then
        Log:e("Badge: Color is null %s", slot:GetName())
        return
    end
    badge:SetTextColor(color.r, color.g, color.b, color.a)
end

function Badge:UpdateMissing()
    local missingCache = Table:Copy(missing)
    missing = {}
    timer = nil
    for _, slot in pairs(missingCache) do
        self:Update(slot)
    end
end

-- Ace3 Callbacks
function Badge:OnInitialize()
    Database:InitIfEmpty(ITEM_FILTER_DB, NO_FILTER)
    InsertOpts(BaikItemLevel:GetArgs())
    Log:v("Badge_OnInitialize")
end

function Badge:OnEnable()
    Log:v("Badge_OnEnable")
end

function Badge:OnDisable()
    Log:v("Badge_OnDisable")
end
