local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local BaikItemLevel = BaikCore.BaikItemLevel
local Assert = BaikCore.Assert
local Database = BaikCore.Database
local Log = BaikCore.Log
local Math = BaikCore.Math
local Ui = BaikCore.Ui

local OrderNumber = BaikItemLevel.OrderNumber

-- Add Module to Addon
local Color = BaikItemLevel:NewModule("Color", "AceEvent-3.0")
BaikItemLevel.Color = Color

-- Constants
local ILVL_COLOR_DB = "item_level_color"

local STYLE_OPTS = {
    "Item Rarity",
    "Relative iLvl"
}

local RARITY_IDX = 1
local RELATIVE_IDX = 2

local COLOR_STYLE_DB = "ilvl_color_style"
local STYLE_DEFAULT = RELATIVE_IDX
local COLOR_RANGE_DB = "ilvl_color_range"
local RANGE_DEFAULT = 30

-- Private Methods
local function GetRange()
    return Database:Get(COLOR_RANGE_DB)
end

local function IsRarityStyle()
    return Database:Get(COLOR_STYLE_DB) == RARITY_IDX
end

local function IsRelativeStyle()
    return Database:Get(COLOR_STYLE_DB) == RELATIVE_IDX
end

local function CreateColorFromQuality(quality)
    local red, green, blue = GetItemQualityColor(quality)

    return Ui:CreateColor(red, green, blue, 1)
end

local function GetQualityColor(itemLink)
    Assert:String(itemLink)

    local itemQuality = select(3, GetItemInfo(itemLink))

    return CreateColorFromQuality(itemQuality)
end

local function GetRelativeColor(itemLink, itemLevel)
    local avgColor = select(2, GetAverageItemLevel())
    local deltaLevel = itemLevel - avgColor
    local range = GetRange()
    deltaLevel = Math:Clamp(deltaLevel, -range, range)

    local itemQuality = math.floor(6 * (deltaLevel + range) / (2 * range))

    return CreateColorFromQuality(itemQuality)
end

local function InitDb()
    Database:InitIfEmpty(COLOR_STYLE_DB, STYLE_DEFAULT)
    Database:InitIfEmpty(COLOR_RANGE_DB, RANGE_DEFAULT)
end

-- Color API
local function InsertOpts(args)
    local colorHeader = {
        name = "Color",
        type = "header",
        order = OrderNumber:GetNext(),
    }
    local colorType = {
        name = "Color Style",
        type = "select",
        values = STYLE_OPTS,
        order = OrderNumber:GetNext(),
        get = function(...)
            return Database:Get(COLOR_STYLE_DB)
        end,
        set = function(self, key, value)
            local curStyle = Database:Get(COLOR_STYLE_DB)

            if key ~= curStyle then
                Database:Set(COLOR_STYLE_DB, key)
                Color:SendMessage(BaikItemLevel.MESSAGE)
            end
        end
    }
    local colorRange = {
        name = "Color Range",
        type = "range",
        min = 10,
        max = 100,
        step = 1,
        order = OrderNumber:GetNext(),
        disabled = IsRarityStyle,
        get = GetRange,
        set = function(self, value, ...)
            Assert:Number(value)
            Database:Set(COLOR_RANGE_DB, value)
            Color:SendMessage(Color.MESSAGE)
        end
    }
    args["color_header"] = colorHeader
    args["color_type"] = colorType
    args["color_range"] = colorRange
end

function Color:Get(itemLink, itemLevel)
    Assert:String(itemLink)
    Assert:Number(itemLevel)
    if IsRelativeStyle() then
        return GetRelativeColor(itemLink, itemLevel)
    elseif IsRarityStyle() then
        return GetQualityColor(itemLink)
    end

    return Ui:CreateColor(1, 1, 1, 1)
end

-- Ace3 Callbacks
function Color:OnInitialize()
    InitDb()
    InsertOpts(BaikItemLevel:GetArgs())
    Log:v("Color_OnInitialize")
end

function Color:OnEnable()
    Log:v("Color_OnEnable")
end

function Color:OnDisable()
    Log:v("Color_OnDisable")
end
